module StaticModel
  class Base
    include Virtus.model

    def id
      defined?(super) ? super : __send__(self.class.primary_key)
    end

    def self.primary_key
      @primary_key ||= superclass == Base ? "id".freeze : superclass.primary_key
    end

    def self.primary_key=(primary_key)
      @primary_key = primary_key
    end

    def self.data_path
      return if self == Base

      @data_path ||= superclass.data_path || default_data_path
    end

    def self.data_path=(data_path)
      @data_path = data_path
    end

    def self.default_data_path
      File.join(StaticModel.base_data_path, "#{name.demodulize.tableize}.yml")
    end

    def self.all
      @records ||= load_records.freeze
    end

    def self.where(criteria = {})
      all.select do |record|
        criteria.all? do |attribute, value|
          record.respond_to?(attribute) && record.__send__(attribute) == value
        end
      end
    end

    def self.find(id)
      find_by!(primary_key => id)
    end

    def self.find_by!(*args)
      find_by(*args) || raise(NotFound)
    end

    def self.find_by(criteria = {})
      where(criteria).first
    end

    def self.exists?(criteria = {})
      if criteria.is_a?(Hash)
        !!find_by(criteria)
      else
        !!find_by(primary_key => criteria)
      end
    end

    def self.load_records
      data = YAML.load_file(data_path)
      data.map { |record_attributes| new(record_attributes).freeze }
    end
  end
end
